/***************************************
*           Eggtray-Volume             *
*                                      *
* Volume setter that runs in the tray  *
* using eggtray                        *
* (it will work in icewmtray)          *
*                                      *
* by Pizzasgood                        *
* with functions from setvol           *
***************************************/


#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <linux/soundcard.h>
#include <gtk/gtk.h>
#include "eggtrayicon.h"
#include "eggtrayicon.c"



void update_volume( GtkAdjustment *adjustment,
					gpointer 	  window )
{
	GtkTooltips *tooltip;
	int volume;
	char tooltext[4];

	volume = -1 * adjustment->value;
	setvol(volume);
	
	sprintf(tooltext, "%d", volume);

	
	tooltip = gtk_tooltips_new();
	gtk_tooltips_set_tip( tooltip,
						  window,
						  tooltext,
						  NULL );
}

/* This is a callback function. The data arguments are ignored
 * in this example. More on callbacks below. */
void action( GtkWidget *widget,
			 GdkEvent  *event,
			 gpointer   window )
{
	GtkWidget *window2, *scale;
	GtkObject *adjustment;
	int volume;
	volume = run_query(0);
	
	
	/* set up the numbers to use with the slider */
	adjustment = gtk_adjustment_new( -1 * volume,
									 -100,
									 0,
									 1,
									 10,
									 0);

	/* set up the slider-bar */
	scale = gtk_vscale_new( adjustment );

	/* enables or disables the volume above the slider */
	gtk_scale_set_draw_value( scale,
							  TRUE );
	/* makes it use whole numbers */
	gtk_scale_set_digits( scale,
						  0 );

	/* sets up a window and puts the slider in it */
	window2 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	
	gtk_widget_set_usize( window2,
                          100,
                          200 );
	gtk_widget_set_uposition( window2,
							  gdk_screen_width()-125,
							  gdk_screen_height()-275 );

	gtk_widget_show( window2 );
	gtk_container_add( GTK_CONTAINER(window2), scale );
	gtk_widget_show( scale );

	gtk_signal_connect (GTK_OBJECT (adjustment), "value_changed",
						GTK_SIGNAL_FUNC (update_volume), window);

	
}


gint delete_event( GtkWidget *widget,
				   GdkEvent  *event,
				   gpointer   data )
{
	/* If you return FALSE in the "delete_event" signal handler,
	 * GTK will emit the "destroy" signal. Returning TRUE means
	 * you don't want the window to be destroyed.
	 * This is useful for popping up 'are you sure you want to quit?'
	 * type dialogs. */

	g_print ("delete event occurred\n");

	/* Change TRUE to FALSE and the main window will be destroyed with
	 * a "delete_event". */

	return(FALSE);
}

/* Another callback */
void destroy( GtkWidget *widget,
			  gpointer   data )
{
	gtk_main_quit();
}




/* These are some funtions from setvol that will let this set the volume */

/* initialize variables and functions*/
int fd;

void open_mixer(void);
void set_value(int channel, int level1, int level2);
int run_query(int channel);
int is_stereo(int channel);
int is_supported(int channel);
int conv_int(char *arg);
void usage(void);


void open_mixer(void)
{
  if ((fd = open("/dev/mixer", 0)) == -1)
    {
      fprintf(stderr, "\nEither /dev/mixer isn't there, or I can't access it.\n");
      fprintf(stderr, "If the latter is the case, chmod 666 /dev/mixer\n\n");
      exit(2);
    }
}

void set_value(int channel, int level1, int level2)
{
  int level;

  if (is_stereo(channel))
    {
      level2 = level2 << 8;
      level = level1 | level2;
    }
  else
    level = level1;
  ioctl(fd, MIXER_WRITE(channel), &level);
}

int run_query(int channel)
{
  open_mixer();
  int level, level1, level2;

  ioctl(fd, MIXER_READ(channel), &level);
  level1 = level&255; 

  close(fd);
  return level1;
}

int is_stereo(int channel)
{
  int devmask;

  ioctl(fd, MIXER_READ(SOUND_MIXER_STEREODEVS), &devmask);
  if (!(devmask & (1 << channel)))
    return 0;
  else
    return 1;
}

int is_supported(int channel)
{
  int devmask;

  ioctl(fd, MIXER_READ(SOUND_MIXER_DEVMASK), &devmask);
  if (!(devmask & (1 << channel)))
    return 0;
  else
    return 1;
}


/* this one does the heavy lifting */
int setvol( int voltouse )
{
	
  int channel;
  int level1, level2;

  open_mixer();
      channel = 0;
      if (!is_supported(channel))
	{
	  fprintf(stderr, "Device %d is not supported by this system\n", channel);
	  close(fd);
	  exit(3);
	}
      level1 = voltouse;
      if (level1 < 0) level1 = 0;
      if (level1 > 100) level1 = 100;
	  level2 = level1;
      set_value(channel, level1, level2);

  close(fd);

    return 0;
	
	
	
}






int main( int   argc,
		  char *argv[] )
{
	/* GtkWidget is the storage type for widgets */
 
	GtkWidget *window, *window2, *pixmapwid, *button, *scale;
	GdkPixmap *pixmap;
	GdkBitmap *mask;
	GtkStyle *style;
	GtkObject *adjustment;
	GtkTooltips *tooltip;
	static EggTrayIcon *egg = NULL;
	int vol = run_query(0);
	char tooltext[4];
  	sprintf(tooltext, "%d", vol);

   
	/* This is called in all GTK applications. Arguments are parsed
	 * from the command line and are returned to the application. */
	gtk_init(&argc, &argv);
	
	/* create a new window */
	window = egg_tray_icon_new("Volume");
	
	/* When the window is given the "delete_event" signal (this is given
	 * by the window manager, usually by the "close" option, or on the
	 * titlebar), we ask it to call the delete_event () function
	 * as defined above. The data passed to the callback
	 * function is NULL and is ignored in the callback function. */
	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
						GTK_SIGNAL_FUNC (delete_event), NULL);
	
	/* Here we connect the "destroy" event to a signal handler.  
	 * This event occurs when we call gtk_widget_destroy() on the window,
	 * or if we return FALSE in the "delete_event" callback. */
	gtk_signal_connect (GTK_OBJECT (window), "destroy",
						GTK_SIGNAL_FUNC (destroy), NULL);
	
	/* Sets the border width of the window. */
	gtk_container_set_border_width (GTK_CONTAINER (window), 1);
	
	gtk_widget_show( window );
 
	
	
 
	 /* load a pixmap from a file */
	pixmap = gdk_pixmap_create_from_xpm( window->window, &mask,
										 &style->bg[GTK_STATE_NORMAL],
										 "/usr/share/mini-icons/mini-sound.xpm" );
//										 "/usr/local/lib/X11/mini-icons/mini-volume.xpm" );
	pixmapwid = gtk_pixmap_new( pixmap, mask );
	gtk_container_add( GTK_CONTAINER(window), pixmapwid );

	gtk_widget_show( pixmapwid );   
   
	/* when the image/window is clicked */
	gtk_widget_add_events(window, GDK_BUTTON_PRESS_MASK);
	gtk_signal_connect( GTK_OBJECT(window), "button_press_event",
						GTK_SIGNAL_FUNC(action), window );
	
	tooltip = gtk_tooltips_new();
	gtk_tooltips_set_tip( tooltip,
						  window,
						  tooltext,
						  NULL );


	
	/* All GTK applications must have a gtk_main(). Control ends here
	 * and waits for an event to occur (like a key press or
	 * mouse event). */
	gtk_main ();
	
	return(0);
}
/* example-end */
